package sub_arctic.anim;

import sub_arctic.input.event;
import sub_arctic.lib.interactor;
import sub_arctic.input.focusable;

/**
 * This is the input protocol interface for specifying that an object is
 * animatable. Keep in mind that the object need <b>not</b>
 * be an interactor, although this is usually the case. Objects
 * other than interactors may need general purpose timing and
 * this can be used for such a timing need. <P>
 */
public interface animatable extends focusable {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This gets called on your interactor when a transition gets started.
   * The start_obj is the result of passing the start time (0.0) through
   * your pacing function and then through the trajectory. Generally, this
   * results in a Point, but other objects are possible. The user_info
   * object is the object you supplied at the time you set yourself into
   * the animation focus set.  This routine will be called for all transitions
   * whose scheduled start time has past, even if there is not enough time
   * to deliver any steps or if the end time has also passed.<p>
   * 
   * @param transition trans the transition being performed.
   * @param trajectory traj the trajectory which is being used for this 
   *                   transition.
   * @param double start_t the amount of time (in the range [0.0, 1.0)) which 
   *                       has already elapsed in this interval.
   * @param Object start_obj the start time mapped through the trajectory
   * @param event e the animation event causing this transition to be started
   * @param Object user_info the object you supplied to the animation agent 
   *                         when you entered its focus set (by scheduling a 
   *                         transition)
   */
  public void start_transition(transition trans, trajectory traj,
			       double start_t, Object start_obj, event e,
			       Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This called on your object to deliver one step of the animation.
   * As with start_transition(), the start_object and end_object are the 
   * result of passing the current time start and end time through your pacing 
   * function then through your trajectory.  Note: start_transition() and 
   * end_transition() will always be called, but steps may not be delivered if
   * there is not enough time.<p>
   *
   * @param transition trans the transition being performed
   * @param trajectory traj the trajectory which is being used for this 
   *                        transition
   * @param double start_t the amount of time (in the range [0.0, 1.0)) at the 
   *                       beginning of this interval
   * @param Object start_obj the start time mapped through the trajectory
   * @param double end_t the amount of time (in the range [0.0, 1.0)) at the 
   *                     end of this interval
   * @param Object end_obj the end time mapped through the trajectory
   * @param event e the animation event causing this transition to be started
   * @param Object user_info the object you supplied to the animation agent 
   *                         when you entered its focus set (by scheduling a 
   *                         transition)
   */
  public void transition_step(transition trans, trajectory traj,
			      double start_t, Object start_obj, 
			      double end_t, Object end_obj,
			      event e, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is the last call for a transition and indicates that the animation
   * transition is completed. Like the transition_step it gives you the start 
   * and end objects based on your trajectory and pacing function.  This will 
   * include the end of the time interval.  This routine is always invoked 
   * (after a call to start_transition()) for transitions whose end time has 
   * passed, even if there was not enough time for any steps.<p>
   *
   * @param transition trans the transition being performed
   * @param trajectory traj the trajectory which is being used for this 
   *                        transition
   * @param double start_t the amount of time (in the range [0.0, 1.0)) at the 
   *                       beginning of this interval
   * @param Object start_obj the start time mapped through the trajectory
   * @param double end_t the amount of time (1.0) at the end of this interval
   * @param Object end_obj the end time mapped through the trajectory
   * @param event e the animation event causing this transition to be started
   * @param Object user_info the object you supplied to the animation agent 
   *                         when you entered its focus set (by scheduling a 
   *                         transition)
   */
  public void end_transition(transition trans, trajectory traj,
			     double start_t, Object start_obj,
			     double end_t, Object end_obj,
			     event e, Object user_info);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
