package sub_arctic.input;

import sub_arctic.lib.interactor;
import sub_arctic.lib.manager;


/** 
 * This class implements an agent which is a hybrid positional/focus agent.
 * It basically accepts a positional press over a properly typed object (an
 * interactor which implements inout_press_draggable) and hands it off to 
 * (i.e., makes it the focus of) the inout_drag agent.
 *
 * @see sub_arctic.input.pressable
 * @see sub_arctic.input.inout_draggable
 * @see sub_arctic.input.inout_drag_focus_agent
 *
 * @author Scott Hudson
 */
public class inout_press_drag_agent extends dispatch_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Simple constructor */
  public inout_press_drag_agent()
    {
      /* nothing to do here */
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Indicate that we are only interested in press of the mouse button. 
   * @param event evt the event we are expressing interest or non-interest in.
   * @return boolean indicating whether we want the event.
   */
  public boolean event_is_useful(event evt)
    {
      return evt.id() == event.MOUSE_DOWN;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Attempt to perform event dispatch to the given object.  If the object
   *  is inout_press_draggable, then we make it the inout_drag focus and we 
   *  are done. 
   *
   * @param event      evt       the event being dispatched.
   * @param Object     user_info ignored (since this is a focus policy).
   * @param interactor to_obj    ignored (since this is a focus policy).
   * @param int        seq_num   ignored (since this is a focus policy).
   * @return boolean indicating whether the input was dispatched and consumed.
   */
  public boolean dispatch_event(
    event            evt,
    Object           user_info,
    interactor       to_obj,
    int              seq_num)
    {
      /* see if it wants this kind of input */
      if (to_obj instanceof inout_press_draggable)
	{
	  /* make the object the inout_drag focus, passing an object_holder  
	   * that has the original position user_info in it as the new 
	   * user_info object.
	   */
	  /* put the event into the local coords of the object */
	  evt.global_to_local(to_obj);
	  
	  /* make the object the inout_drag_focus */
	  manager.
	    inout_drag_focus.set_focus_to((inout_press_draggable)to_obj, 
					  evt, new object_holder(user_info));
	  return true;
	}
      else
        return false;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
