package sub_arctic.input;

import sub_arctic.lib.interactor;
import sub_arctic.lib.interactor_consts;
import java.awt.Point;

/** 
 * Focus agent that implements move-drag.  Move drag is a drag designed for
 * moving objects around and is the most sophisticated of the drag protocols.
 * The move_drag protocol provides for drag_start(), drag_feedback(), and
 * drag_end() methods.  The parameters passed to these supply an x,y position
 * in the parent's coordinate system which can be used to directly set the 
 * position of the object being dragged.<p>
 * 
 * Move-drag also implements a filtering (or limiting) process based on feature
 * points of an object.  Each object implements a set of feature points that
 * represent interesting locations within it (by default these include the 
 * four corners and the center, plus the point at which the object was 
 * "grabbed" at the beginning of the drag).  Move_drag_filters objects can
 * can be used to modify the set of point that a designated feature point
 * passes through.  This is most often used to limit the allowable drag
 * locations.  For example, to limit the drag of an object to stay within
 * the bounds of its parent.  Note: base_interactor implements 
 * move_drag_filter, so often objects act as their own drag filter.  
 * std_drag_filters provides a useful collection of filter functions for this
 * purpose.
 *
 * @see sub_arctic.input.move_draggable
 * @see sub_arctic.input.move_drag_filter
 * @see sub_arctic.lib.base_interactor#filter_pt
 * @see sub_arctic.input.std_drag_filters
 * @author  Scott Hudson
 */
public class move_drag_focus_agent extends single_focus_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Default constructor */
  public move_drag_focus_agent()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Method to limit the objects allowable as members of the focus set.
   * In this case we only allow things which implement move_draggable.
   * @param candidate_obj the object we are deciding upon
   * @return a boolean indicating if the object is acceptable
   */
  public boolean allowable_focus(focusable candidate_obj)
    {
      /* must be a move_draggable */
      return candidate_obj instanceof move_draggable;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Method to determine if this agent needs to see a given event.  In this
   * case we are interested in MOUSE_DRAG, MOUSE_MOVE and MOUSE_UP only.
   * @param evt the event in question
   * @boolean indication of whether the event should be passed to this agent
   */
  public boolean event_is_useful(event evt)
    {
      /* interested in drag, move, and button up */
      return evt.id() == event.MOUSE_DRAG || evt.id() == event.MOUSE_MOVE || 
	     evt.id() == event.MOUSE_UP;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Index of the feature point we are dragging based on.  The feature
   * point is used to filter the dragging.  In particular, the position of
   * the feature point is what is passed to the filter applied to the
   * drag.  The result of the filter is used to calculate a top left value
   * which is passed to the drag protocol routines.  This position is such
   * that it places the feature point at its filtered position.  Standard
   * indexes for the four corners and center are defined in 
   * sub_arctic.lib.interactor_consts.  A value of FEATURE_NONE (== -1) can
   * be used to denote the "drag point" -- the point within the object that
   * that the initial mouse-down occured at, which would also normally be the 
   * position of the cursor.
   *
   * @see sub_arctic.lib.interactor_consts
   * @see sub_arctic.lib.base_interactor#feature_point
   * @see sub_arctic.lib.base_interactor#drag_feature_point
   */
  protected int _drag_feature_point = interactor_consts.FEATURE_TOP_LEFT;

  /** 
   * Index of the feature point we are dragging based on.  The feature
   * point is used to filter the dragging.  In particular, the position of
   * the feature point is what is passed to the filter applied to the
   * drag.  The result of the filter is used to calculate a top left value
   * which is passed to the drag protocol routines.  This position is such
   * that it places the feature point at its filtered position.  Standard
   * indexes for the four corners and center are defined in 
   * sub_arctic.lib.interactor_consts.  A value of FEATURE_NONE (== -1) can
   * be used to denote the "drag point" -- the point within the object that
   * that the initial mouse-down occured at, which would also normally be the 
   * position of the cursor.
   *
   * @return the index of the feature point in use for the current drag
   */
  public int drag_feature_point() {return _drag_feature_point;}

  /** 
   * Set the index of the feature point we are dragging based on.  The feature
   * point is used to filter the dragging.  In particular, the position of
   * the feature point is what is passed to the filter applied to the
   * drag.  The result of the filter is used to calculate a top left value
   * which is passed to the drag protocol routines.  This position is such
   * that it places the feature point at its filtered position.  Standard
   * indexes for the four corners and center are defined in 
   * sub_arctic.lib.interactor_consts.  A value of FEATURE_NONE (== -1) can
   * be used to denote the "drag point" -- the point within the object that
   * that the initial mouse-down occured at, which would also normally be the 
   * position of the cursor.
   *
   * @param indx the new feature point index
   */
  public void set_drag_feature_point(int indx) {_drag_feature_point = indx;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Object which acts as drag filter.  This filter object transforms the
   * drag point (which is the position of the "feature point" within the 
   * object -- see drag_feature_point()) into other points and is used to 
   * limit the area of the drag or produce other special drag effects.  If 
   * this is null, no filtering is done.
   *
   * @see sub_arctic.input.move_drag_focus_agent#drag_feature_point
   */
  protected move_drag_filter _filter_obj = null;

  /* note: if we go to a multi-focus drag, this needs to be a list of lookup 
   *       table so we have a per-focus filter. */

  /** 
   * Object which acts as drag filter.  This filter object transforms the
   * drag point (which is the position of the "feature point" within the 
   * object -- see drag_feature_point()) into other points and is used to 
   * limit the area of the drag or produce other special drag effects.  If 
   * this is null, no filtering is done.
   *
   * @return the current filter object
   */
  public move_drag_filter filter_obj() {return _filter_obj;}

  /** 
   * Set the object which acts as drag filter.  This filter object 
   * transforms the drag point (which is the position of the "feature point" 
   * within the object -- see drag_feature_point()) into other points and is 
   * used to limit the area of the drag or produce other special drag effects.
   * If this is set to null, no filtering is done.
   *
   * @param fo the new filter object
   */
  public void set_filter_obj(move_drag_filter fo) {_filter_obj = fo;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override set_focus_to to record the focus object as the filter object.
   * If the focus object implements move_drag_filter, then we use it as the
   * filter.  Otherwise we set the filter to null (which means no filtering).
   * We also record the feature point of the object that is to be used for
   * the drag (as returned by its drag_feature_point() method).  
   *
   * @param to_obj    the object which is the new drag focus
   * @param evt       the event which "caused" the focus to occur
   * @param user_info information that is to be passed to the object whenever 
   *                  input is dispatched to it
   */
  public void set_focus_to(focusable to_obj, event evt, Object user_info) 
{
      /* let the superclass to the normal thing */
      super.set_focus_to(to_obj, evt, user_info);

      /* if the object is a move_drag_filter set it as our filter */
      if (to_obj instanceof move_drag_filter)
	{
	  set_filter_obj((move_drag_filter)to_obj);
	}
      else
	set_filter_obj(null);

      /* record the object's feature point index */
      set_drag_feature_point(((interactor)to_obj).drag_feature_point());
    }

   //had:
   //* @exception sub_arctic.exception.bad_value if the object is not eligible
   //*                  to be the focus (i.e., is not a move_draggable object)

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Version of set_focus_to which allows separate move_drag_filter object. 
   * 
   * @param to_obj    the object which is the new drag focus
   * @param filt_obj  the filter to be applied to the drag
   * @param evt       the event which "caused" the focus to occur
   * @param user_info information that is to be passed to the object whenever 
   *                  input is dispatched to it
   */
  public void set_focus_to(
    focusable        to_obj, 
    move_drag_filter filt_obj, 
    event            evt, 
    Object           user_info) 
{
      /* let the superclass to the normal thing */
      super.set_focus_to(to_obj, evt, user_info);

      /* set the filter and feature point */
      set_filter_obj(filt_obj);
      set_drag_feature_point(((interactor)filt_obj).drag_feature_point());
    }

   //had:
   //* @exception sub_arctic.exception.bad_value if the object is not eligible
   //*                  to be the focus (i.e., is not a move_draggable object)

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Version of set_focus_to which allows separate move_drag_filter object and
   * feature point specification.  If this constructor is used, the 
   * drag_feature_point() method of the object is ignored in favor of the 
   * point given here.
   *
   * @param to_obj    the object which is the new drag focus
   * @param filt_obj  the filter to be applied to the drag
   * @param f_pt      the index of the feature point to be used for this drag
   * @param evt       the event which "caused" the focus to occur
   * @param user_info information that is to be passed to the object whenever 
   *                  input is dispatched to it
   */
  public void set_focus_to(
    focusable    to_obj, 
    move_drag_filter filt_obj, 
    int          f_pt,
    event        evt, 
    Object       user_info) 
{
      /* let the superclass to the normal thing */
      super.set_focus_to(to_obj, evt, user_info);

      /* set the filter and feature point */
      set_filter_obj(filt_obj);
      set_drag_feature_point(f_pt);
    }

   //had:
   //* @exception sub_arctic.exception.bad_value if the object is not eligible
   //*                  to be the focus (i.e., is not a move_draggable object)

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Start x position of the current drag (in global coordinates) */
  protected int start_x = 0;

  /** Start y position of the current drag (in global coordinates) */
  protected int start_y = 0;

  /** Grab x position (start position in local coordinates).  This is where
   *  the cursor was positioned relative to the object when the mouse button
   *  went down to initiate the drag.
   */
  protected int grab_x = 0;

  /** Grab y position (start position in local coordinates).  This is where
   *  the cursor was positioned relative to the object when the mouse button
   *  went down to initiate the drag.
   */
  protected int grab_y = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Filter a drag point.  If there were no filtering, then the point given 
   * would correspond to the grab point within the object (expressed in the
   * parent coordinate system).  Based on this we compute where the  object's 
   * feature point would be.  We filter this point, then calculate the 
   * location of the top-left corner of the object that places the feature
   * point at the filtered location.  This top-left corner is what is finally
   * returned.
   *
   * @param for_obj  The object we are filtering the point for 
   * @param pt       The actual point being filtered (in parent's coordinates)
   * @return the filtered point (in parent's coordinates)
   */
  protected Point filter_drag_pt(interactor for_obj, Point pt)
    {
      int   pt_indx;
      Point feature_pt, result_pt;
      int x = pt.x;
      int y = pt.y;

      /* get the feature point, defaulting to the grab point */ 
      pt_indx = drag_feature_point();
      if (pt_indx == interactor_consts.FEATURE_NONE)
          feature_pt = new Point(grab_x, grab_y);
      else
        feature_pt = for_obj.feature_point(drag_feature_point());

      /* calculate the feature point location */
      x = x - grab_x + feature_pt.x;
      y = y - grab_y + feature_pt.y;

      /* filter it */
      if (filter_obj() != null)
	{
	  result_pt = filter_obj().filter_pt(new Point(x,y),for_obj,feature_pt);
	}
      else
	{
	  /* no filter */
	  result_pt = new Point(x,y);
	}

      /* calculate final top-left to put feature point there */
      result_pt.x -= feature_pt.x;
      result_pt.y -= feature_pt.y;

      return result_pt;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override the normal inform_focus_enter to deliver the drag_start().
   *
   * @param obj       the object we are dragging
   * @param evt       the event that "caused" the start of drag (typically 
   *                  a mouse button down event)
   * @param user_info an uninterpreted object to be passed to the object
   */
  protected void inform_focus_enter(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      move_draggable drag_obj;
      Point          glob_pt, pt;
      event          cur_event;

      /* do a drag_start rather than the default focus_set_enter() */

      /* retype obj to what it is supposed to be */
      drag_obj = (move_draggable)obj;

      /* make a copy of the event */
      cur_event = new event(evt);

      /* put the event into local coords of the object and save grab pt */
      cur_event.global_to_local(drag_obj);
      grab_x = cur_event.local_x();
      grab_y = cur_event.local_y();

      /* Compute position in parent, then filter it */
      pt = filter_drag_pt(drag_obj, drag_obj.into_parent(grab_x,grab_y));

      /* put the point back into global coords, then save it as start point */
      glob_pt = drag_obj.local_to_global(drag_obj.into_local(pt));
      start_x = glob_pt.x;
      start_y = glob_pt.y;

      /* dispatch the start of the drag to the object */
      drag_obj.drag_start(cur_event, pt.x, pt.y, grab_x, grab_y, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override the normal inform_focus_exit to deliver the drag_end().
   *
   * @param obj       the object we are dragging
   * @param evt       the event that "caused" the end of drag (typically
   *                  a mouse button up event)
   * @param user_info an uninterpreted object to be passed to the object
   */
  protected void inform_focus_exit(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      move_draggable drag_obj;
      event          cur_event;
      Point          pt, start_pt;

      /* get object into proper type */
      drag_obj = (move_draggable)obj;

      /* end the drag */

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* Compute position in parent, then filter it */
      pt = filter_drag_pt(drag_obj,
	        drag_obj.into_parent(cur_event.local_x(),cur_event.local_y()));

      /* put start point into parent's coords */
      start_pt = drag_obj.global_to_local(start_x, start_y);
      start_pt = drag_obj.into_parent(start_pt);

      /* dispatch drag_end to the object */
      drag_obj.drag_end(cur_event, pt.x, pt.y, 
			     start_pt.x, start_pt.y, grab_x, grab_y, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch an event under this protocol.  This will send either a 
   * drag_feedback() or drag_end() to the object.
   *
   * @param evt       the event being dispatched (already approved by 
   *                  event_is_useful())
   * @param user_info an uninterpreted info object to be passed to the 
   *                  dragged object
   * @param to_obj    for focus agents this is always null
   * @param seq_num   for focus agents this is ignored
   * @return value indicating if the event was consumed 
   */
  public boolean dispatch_event(
    event      evt,
    Object     user_info,
    interactor to_obj,
    int        seq_num) 
{
      move_draggable drag_obj;
      Object         drag_info;
      event          cur_event;
      int            prev_off_x, prev_off_y, off_x, off_y;
      boolean        result;
      Point          pt, start_pt;

      /* extract our focus item and if there is none, bail out now */
      drag_obj = (move_draggable)_focus_set.elementAt(0);
      if (drag_obj == null) return false;
      drag_info = _user_info_set.elementAt(0);

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* If this is end, clear the focus and let inform_focus_exit do work */
      if (cur_event.id() == event.MOUSE_UP)
	{
	  clear_focus(cur_event);
	  return true;
	}

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* Compute position in parent, then filter it */
      pt = filter_drag_pt(drag_obj,
		 drag_obj.into_parent(cur_event.local_x(),cur_event.local_y()));

      /* put start point into parent's coords */
      start_pt = drag_obj.global_to_local(start_x, start_y);
      start_pt = drag_obj.into_parent(start_pt);

      /* dispatch drag_feedback to the object */
      return drag_obj.drag_feedback(cur_event, pt.x, pt.y, 
			     start_pt.x, start_pt.y, grab_x, grab_y, drag_info);
    }
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
