package sub_arctic.input;

import sub_arctic.lib.interactor;
import java.awt.Point;

/** 
 * Focus dispatch agent for misc. dragging.  This agent dispatches under the
 * simple_drag input protocol which is designed for generic dragging (i.e.,
 * anything not handled by one of the more specifically targeted drag 
 * protocols).  Its particularly suitable for dragging components that are
 * inside the object itself (such as the thumb of a slider).
 */
public class simple_drag_focus_agent extends single_focus_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Simple constructor. */ 
  public simple_drag_focus_agent()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Indicate if the given object is suitable for inclusion in the focus set
   * of this object.  In this case the object must be a simple_draggable.
   * 
   * @param focusable candidate_obj the object we are testing suitability for.
   * @return boolean indicating whether the object is suitable.
   */
  public boolean allowable_focus(focusable candidate_obj)
    {
      /* must be a simple_draggable */
      return candidate_obj instanceof simple_draggable;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Indicate if the given event should be passed to this agent.  In this 
   * case we are interested in MOUSE_DRAG, MOUSE_MOVE, or MOUSE_UP events.
   * 
   * @param event evt the event we are testing.
   * @return boolean indicating whether we want this event.
   */
  public boolean event_is_useful(event evt)
    {
      /* interested in drag, move, and button up */
      return evt.id() == event.MOUSE_DRAG || evt.id() == event.MOUSE_MOVE || 
	     evt.id() == event.MOUSE_UP;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Do the work needed when an object goes in the focus set.  In this case
   * we send the object a drag_start().
   *
   * @param focusable obj       the object entering the focus set.
   * @param event     evt       the event "causing" that entry.
   * @param Object    user_info the user info that should be sent to the object.
   */
  protected void inform_focus_enter(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      simple_draggable drag_obj;
      event            cur_event;

      /* do a drag_start rather than the default focus_set_enter() */

      /* retype obj to what it is supposed to be */
      drag_obj = (simple_draggable)obj;

      /* keep a copy of the old event */
      cur_event = new event(evt);

      /* put the event into local coords of the object */
      cur_event.global_to_local(drag_obj);

      /* dispatch the start of the drag to the object */
      drag_obj.drag_start(cur_event, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Do the work needed when an object comes out of the focus set.  In this case
   * we send the object an drag_end().
   *
   * @param focusable obj       the object leaving the focus set.
   * @param event     evt       the event "causing" that exit.
   * @param Object    user_info the user info that should be sent to the object.
   */
  protected void inform_focus_exit(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      simple_draggable drag_obj;
      event            cur_event;

      /* get object into proper type */
      drag_obj = (simple_draggable)obj;

      /* end the drag */

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* dispatch drag_end to the object */
      drag_obj.drag_end(cur_event, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Attempt to dispatch an event via this agent.
   *
   * @param  event      evt       the event being dispatched.
   * @param  Object     user_info ignored (because this is a focus agent).
   * @param  interactor to_obj    ignored (because this is a focus agent).
   * @param  int        seq_num   ignored (because this is a focus agent).
   */
  public boolean dispatch_event(
    event      evt,
    Object     user_info,
    interactor to_obj,
    int        seq_num) 
{
      simple_draggable drag_obj;
      Object           drag_info;
      event            cur_event;

      /* extract our focus item and if there is none, bail out now */
      drag_obj = (simple_draggable)_focus_set.elementAt(0);
      if (drag_obj == null) return false;
      drag_info = _user_info_set.elementAt(0);

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* If this is end, clear the focus and let inform_focus_exit do work */
      if (evt.id() == event.MOUSE_UP)
	{
	  clear_focus(cur_event);
	  return true;
	}

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* dispatch drag_feedback to the object */
      return drag_obj.drag_feedback(cur_event, drag_info);
    }

   //had:
   //* @exception general

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
