package sub_arctic.input;

import sub_arctic.lib.interactor;
import sub_arctic.lib.interactor_consts;
import sub_arctic.lib.top_level;
import sub_arctic.lib.interactor_pred;
import sub_arctic.lib.traversal_xform;
import sub_arctic.lib.pred_and;
import sub_arctic.lib.rect_bound_overlap;
import sub_arctic.lib.xform_rect_to_child;
import sub_arctic.lib.manager;

import java.util.Vector;
import java.util.Hashtable;
import java.awt.Point;
import java.awt.Rectangle;

/** 
 * Focus agent that implements snap-dragging.  Snap dragging is an interaction
 * technique where objects being dragged snap to interesting points within
 * objects they are dragged over.  These interesting points typically represent
 * semantically interesting places to "let go of" the dragged object such as
 * positions where they will legally "connect to" other object.  The 
 * determination of what represents an interesting point is done by means
 * of predicates on the objects snapped from (the ones being dragged) and 
 * snapped to (the ones "under" them).  In snap dragging, dragged objects 
 * implement the snap_draggable protocol,  while objects which they are 
 * dragged over, and can be snapped to, implement the snap_targetable 
 * protocol (and generally are called snap targets).<p>
 * 
 * Each dragged object advertises a set of feature points.  By default 
 * base_interactor provides 5: the 4 corners plus the center of the object.
 * Other object types may provide additional or complete different points that
 * suit their semantics.  Each active feature point is eligible to snap to 
 * a snap target object.  Snap target objects are not limited to being points
 * and may implement areas, lines, points, and other geometric forms.  With 
 * each target comes a geometric test to determine if a point is considered 
 * close enough to the target to snap to it (and provide a distance to it). 
 * Snapping occurs by considering active feature point within snapping
 * distance of a target.  For each geometrically possible snap, semantic tests 
 * are performed to determine if the snap is semantically acceptable (i.e., it
 * constitutes an "interesting" snap).  The closest candidate snap that passes
 * semantic tests is performed.  Both dragged and target objects are informed
 * of the snap (an any subsequent unsnap) and given the opportunity to provide 
 * feedback. If snaps are geometrically possible, but none pass the semantic 
 * tests, then starting with the closest each is given the opportunity to make
 * a special "anti-snap".  This is done to provide negative feedback (I kind
 * of error message) which indicates why a geometrically legal snap has been
 * ruled out semantically.<p>
 *
 * Once a snap has been performed, users can reject the snap by moving 
 * the dragged object out of range of the snap.  In addition, the snap can
 * be rejected and disabled (for the duration of the drag operation) by 
 * pressing a designated keyboard key (by default space -- this can be changed
 * with the set_snap_break_key() method) while the snap is in force.<p>
 *
 * Note: in order for snap disabling to occur, this agent needs to be (and 
 * by default is) installed before the text focus agent within the agent list 
 * of the focus policy.
 * 
 * @see snap_draggable
 * @see snap_targetable
 * @author  Scott Hudson
 */
public class snap_drag_agent extends single_focus_agent {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Default constructor */
  public snap_drag_agent()
    {
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Method to limit the objects allowable as members of the focus set.
   * In this case we only allow things which implement snap_draggable.
   *
   * @param candidate_obj the object we are deciding upon
   * @return a boolean indicating if the object is acceptable
   */
  public boolean allowable_focus(focusable candidate_obj)
    {
      /* must be a snap_draggable */
      return candidate_obj instanceof snap_draggable;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * The keyboard event code used to signal that the currently active snap 
   * should be be broken and disabled.  The snap stays disabled until the
   * next snap drag sequence.  Currently this key is <space>.
   */
  protected static int _snap_break_key = (int)' '; 

  /** 
   * The keyboard event code used to signal that the currently active snap 
   * should be be broken and disabled.  The snap stays disabled until the
   * next snap drag sequence.  Currently this key is <space>.
   */
  public static int snap_break_key() {return _snap_break_key;}

  /** 
   * Set the keyboard event code used to signal that the currently active snap 
   * should be be broken and disabled.  The snap stays disabled until the
   * next snap drag sequence.  
   */
  public static void set_snap_break_key(int char_val) 
    {
      _snap_break_key = char_val;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Method to determine if this agent needs to see a given event.  In this
   * case we are interested in MOUSE_DRAG, MOUSE_MOVE and MOUSE_UP all the
   * time and the keyboard events associated for breaking a snap only when
   * we have an active snap or anti_snap.
   *
   * @param evt the event in question
   * @boolean indication of whether the event should be passed to this agent
   */
  public boolean event_is_useful(event evt)
    {
      /* interested in drag, move, and button up */
      return evt.id() == event.MOUSE_DRAG || evt.id() == event.MOUSE_MOVE || 
	     evt.id() == event.MOUSE_UP || 
	     (have_active() && evt.id() == event.KEY_PRESS && 
			       evt.key() == snap_break_key());
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Start x position of the current drag (in global coordinates) */
  protected int start_x = 0;

  /** Start y position of the current drag (in global coordinates) */
  protected int start_y = 0;

  /** Grab x position (start position in local coordinates).  This is where
   *  the cursor was positioned relative to the object when the mouse button
   *  went down to initiate the drag.
   */
  protected int grab_x = 0;

  /** Grab y position (start position in local coordinates).  This is where
   *  the cursor was positioned relative to the object when the mouse button
   *  went down to initiate the drag.
   */
  protected int grab_y = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Set of active feature points of the object being dragged (stored as
   *  a Vector of Point objects).
   */
  protected Vector feature_points = null;

  /** Number of active feature points for the object currently being dragged. */
  protected int num_feature_points = 0;

  /** Min x of feature points */
  protected int min_feature_x = 0;

  /** Min y of feature points */
  protected int min_feature_y = 0;

  /** Width of feature point bounding rectangle */
  protected int feature_rect_w = 0;

  /** Height of feature point bounding rectangle */
  protected int feature_rect_h = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Collect feature point information from a new focus object.  Information
   * is stored in various instance variables for use during the drag.
   * @param interactor obj the object we are collecting feature points from.
   */
  protected void collect_feature_info(interactor obj)
    {
      Point pt;
      int max_feature_x = -99999;
      int max_feature_y = -99999;

      /* allocate a new vector to hold the active points and init stats */
      num_feature_points = 0;
      feature_points = new Vector();
      min_feature_x = 0x7fffffff;
      min_feature_y = 0x7fffffff;

      /* walk down feature points advertised by object and record active ones */
      for (int i = 0; i < obj.num_feature_points(); i++)
	{
	  /* only look at the enabled points */
	  if (obj.feature_point_enabled(i))
	    {
	      /* count it */
	      num_feature_points++;

	      /* extract the point and add to our collection */
	      pt = obj.feature_point(i);
	      feature_points.addElement(pt);

	      /* update min and max statistics */
	      if (pt.x > max_feature_x) max_feature_x = pt.x;
	      if (pt.y > max_feature_y) max_feature_y = pt.y;
	      if (pt.x < min_feature_x) min_feature_x = pt.x;
	      if (pt.y < min_feature_y) min_feature_y = pt.y;
	    }
	}

      /* if we have at least one point, compute bounding box w,h */
      if (num_feature_points > 0)
	{
          feature_rect_w = max_feature_x - min_feature_x;
          feature_rect_h = max_feature_y - min_feature_y;
	}
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override the normal inform_focus_enter to deliver the drag_start().
   *
   * @param obj       the object we are dragging
   * @param evt       the event that "caused" the start of drag (typically 
   *                  a mouse button down event)
   * @param user_info an uninterpreted object to be passed to the object
   */
  protected void inform_focus_enter(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      snap_draggable drag_obj;
      Point          glob_pt;
      event          cur_event;

      /* do a drag_start rather than the default focus_set_enter() */

      /* retype obj to what it is supposed to be */
      drag_obj = (snap_draggable)obj;

      /* gather the feature point stats we will need */
      collect_feature_info(drag_obj);

      /* make a copy of the event */
      cur_event = new event(evt);

      /* put the event into local coords of the object and save grab pt */
      cur_event.global_to_local(drag_obj);
      grab_x = cur_event.local_x();
      grab_y = cur_event.local_y();

      /* put the point back into global coords, then save it as start point */
      glob_pt = drag_obj.local_to_global(0,0);
      start_x = glob_pt.x;
      start_y = glob_pt.y;

      /* dispatch the start of the drag to the object */
      drag_obj.drag_start(cur_event, drag_obj.x(), drag_obj.y(), 
						   grab_x, grab_y, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Override the normal inform_focus_exit to deliver the drag_end().
   *
   * @param obj       the object we are dragging
   * @param evt       the event that "caused" the end of drag (typically
   *                  a mouse button up event)
   * @param user_info an uninterpreted object to be passed to the object
   */
  protected void inform_focus_exit(
    focusable obj, 
    event     evt, 
    Object    user_info)
    {
      snap_draggable drag_obj;
      event          cur_event;
      Point          start_pt, pt, feature_pt;
      int            x,y;

      /* get object into proper type */
      drag_obj = (snap_draggable)obj;

      /* clear out the snap rejection list */
      rejected_snaps = new Hashtable();

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* if we have an active snap we use that for our position */
      if (have_active_snap())
	{
          /* put snap point into drag object's parent's coordinates */ 
          pt = active_snap.target_obj.local_to_global(active_snap.snap_pt);
          pt = drag_obj.global_to_local(pt);
          pt = drag_obj.into_parent(pt);

          /* account for offset of feature point within drag object. */
          feature_pt = drag_obj.feature_point(active_snap.drag_feature);
          x = pt.x - feature_pt.x;
          y = pt.y - feature_pt.y;
	}
      /* otherwise we compute from event */
      else
	{
          /* Compute our new position in parent */
          x = drag_obj.x_into_parent(cur_event.local_x()) - grab_x;
          y = drag_obj.y_into_parent(cur_event.local_y()) - grab_y;
	}

      /* after this we have no active snap or anti-snap */ 
      active_snap = null;
      active_anti_snap = null;
      active_anti_snap_info = null;

      /* put start point into parent's coords */
      start_pt = drag_obj.global_to_local(start_x, start_y);
      start_pt = drag_obj.into_parent(start_pt);

      /* dispatch drag_end to the object */
      drag_obj.drag_end(cur_event, x, y, 
			     start_pt.x, start_pt.y, grab_x, grab_y, user_info);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Distance at which snap points should become active */
  public static int snap_dist() {return 20;}

  /** Square of distance at which snap points should become active */
  public static int snap_dist2() {return snap_dist() * snap_dist(); }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Traversal identifier for the snap bound test traversal */
  protected static final int _bound_traversal = manager.unique_int();


  /** Traversal identifier for the snap bound test traversal */
  public static int bound_traversal() {return _bound_traversal; }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Interactor predicate object to test if an expanded snap region overlaps
   * the bounds of an object.  The parameter to this predicate is a Rectangle
   * object which represents the bound of the feature points of the drag
   * object, expanded by the maximum snap_distance.
   */
  protected static final interactor_pred overlaps = new rect_bound_overlap();

  /** Interactor predicate object to test if an object is a snap_targetable.
   */
  protected static final interactor_pred is_snap_obj = new is_snap_targetable();

  /** Interactor predicate object to test if an object is both a 
   *  snap_targetable object and passes the overlap test. */
  protected static final interactor_pred snap_and_overlap = 
    new pred_and(is_snap_obj, overlaps);

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Traversal transformation object to transform a Rectangle from parent 
   *  to child coordinates.  */
  protected static final traversal_xform rect_to_child = 
    new xform_rect_to_child();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Snap record containing information about the currently active snap
   *  (or null if there is none).
   */
  protected snap_record active_snap = null;

  /** Snap record containing information about the currently active anti-snap
   *  (or null if there is none).
   */
  protected snap_record active_anti_snap = null;

  /** Application specific information about the currently active anti-snap */
  protected Object active_anti_snap_info = null;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Indicate whether we have an active snap */
  public boolean have_active_snap() {return active_snap != null; }

  /** Indicate whether we have an active anti-snap */
  public boolean have_active_anti_snap() {return active_anti_snap != null; }

  /** Indicate whether we have either an active snap or anti_snap */
  public boolean have_active() 
    {return have_active_snap() || have_active_anti_snap(); }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Break the currently active snap.
   * 
   * @param event  evt       the event "causing" the break.
   * @param Object user_info the user info value passed to the object.
   */
  protected void break_active_snap(event evt, Object user_info)
    {
      Point drag_pt, snap_pt, feature_pt;
      event evt_copy;

      /* if we have no snap, do nothing */
      if (!have_active_snap()) return;

      /* put drag_pt into target object's coordinates */
      drag_pt = active_snap.drag_obj.into_local(active_snap.drag_pt);
      drag_pt = active_snap.drag_obj.local_to_global(drag_pt);
      drag_pt = active_snap.target_obj.global_to_local(drag_pt);

      /* make a copy of the event */
      evt_copy = new event(evt);

      /* break from target obj */
      evt_copy.global_to_local(active_snap.target_obj);
      active_snap.target_obj.unsnap_from(
        evt_copy, drag_pt, active_snap.snap_pt, 
        active_snap.drag_obj, active_snap.drag_feature, user_info);

      /* put snap_pt into drag object's parent's coordinates */ 
      snap_pt = active_snap.target_obj.local_to_global(active_snap.snap_pt);
      snap_pt = active_snap.drag_obj.global_to_local(snap_pt);
      snap_pt = active_snap.drag_obj.into_parent(snap_pt);

      /* account for offset of feature point within drag object. */
      feature_pt = active_snap.drag_obj.feature_point(active_snap.drag_feature);
      snap_pt.x -= feature_pt.x;
      snap_pt.y -= feature_pt.y;

      /* break from drag obj */
      evt_copy.global_to_local(active_snap.drag_obj);
      active_snap.drag_obj.unsnap_to(evt_copy, active_snap.drag_pt, snap_pt, 
	active_snap.drag_feature, active_snap.target_obj, user_info);

      active_snap = null;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Break the currently active anti_snap 
   * 
   * @param event  evt       the event "causing" the break.
   * @param Object user_info the user info value passed to the object.
   */
  protected void break_active_anti_snap(event evt, Object user_info)
    {
      Point drag_pt, snap_pt, feature_pt;
      event evt_copy;

      if (!have_active_anti_snap()) return;

      /* put drag_pt into target object's coordinates */
      drag_pt = active_anti_snap.drag_obj.into_local(active_anti_snap.drag_pt);
      drag_pt = active_anti_snap.drag_obj.local_to_global(drag_pt);
      drag_pt = active_anti_snap.target_obj.global_to_local(drag_pt);

      /* make a copy of the event */
      evt_copy = new event(evt);

      /* break from target obj */
      evt_copy.global_to_local(active_anti_snap.target_obj);
      active_anti_snap.target_obj.unanti_snap_from(
        evt_copy, drag_pt, active_anti_snap.snap_pt, active_anti_snap.drag_obj, 
	active_anti_snap.drag_feature, active_anti_snap_info, user_info);

      /* put snap_pt into drag object's parent's coordinates */
      snap_pt = active_anti_snap.target_obj.local_to_global(
						     active_anti_snap.snap_pt);
      snap_pt = active_anti_snap.drag_obj.global_to_local(snap_pt);
      snap_pt = active_anti_snap.drag_obj.into_parent(snap_pt);

      /* account for offset of feature point within drag object. */
      feature_pt = active_anti_snap.drag_obj.feature_point(
					    active_anti_snap.drag_feature);
      snap_pt.x -= feature_pt.x;
      snap_pt.y -= feature_pt.y;

      /* break from drag obj */
      evt_copy.global_to_local(active_anti_snap.drag_obj);
      active_anti_snap.drag_obj.unanti_snap_to( 
	evt_copy, active_anti_snap.drag_pt, snap_pt, 
	active_anti_snap.drag_feature, active_anti_snap.target_obj, 
	active_anti_snap_info, user_info);

      active_anti_snap = null;
      active_anti_snap_info = null;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Activate a new snap.  
   * 
   * @param snap_record snap_rec  record describing the snap we are activating.
   * @param event       evt       event that "caused" the snap.
   * @param Object      user_info the user info passed to the objects.
   * @return boolean indicating if either the drag object or the 
   *                 target object accepted the snap.
   */
  protected boolean make_snap_active(
    snap_record snap_rec, 
    event       evt, 
    Object      user_info)
    {
      boolean result;
      Point drag_pt, snap_pt, feature_pt;
      event evt_copy;

      /* record new active snap */
      active_snap = snap_rec;
      active_anti_snap = null;
      active_anti_snap_info = null;

      /* put snap_pt into drag object's parent's coordinates */
      snap_pt = active_snap.target_obj.local_to_global(active_snap.snap_pt);
      snap_pt = active_snap.drag_obj.global_to_local(snap_pt);
      snap_pt = active_snap.drag_obj.into_parent(snap_pt);

      /* account for offset of feature point within drag object. */
      feature_pt = active_snap.drag_obj.feature_point(active_snap.drag_feature);
      snap_pt.x -= feature_pt.x;
      snap_pt.y -= feature_pt.y;

      /* make copy of event */
      evt_copy = new event(evt);

      /* snap drag obj -- note reverse order from break */
      evt_copy.global_to_local(active_snap.drag_obj);
      result = active_snap.drag_obj.snap_to(evt_copy, active_snap.drag_pt, 
	snap_pt, active_snap.drag_feature, active_snap.target_obj, user_info);

      /* put drag_pt into target object's coordinates */
      drag_pt = active_snap.drag_obj.into_local(active_snap.drag_pt);
      drag_pt = active_snap.drag_obj.local_to_global(drag_pt);
      drag_pt = active_snap.target_obj.global_to_local(drag_pt);

      /* snap target obj */
      evt_copy.global_to_local(active_snap.target_obj);
      result |= active_snap.target_obj.snap_from(
        evt_copy, drag_pt, active_snap.snap_pt, 
        active_snap.drag_obj, active_snap.drag_feature, user_info);

      return result;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Activate a new anti-snap.  
   * 
   * @param snap_record snap_rec  record describing the anti-snap we are 
   *                              activating.
   * @param event       evt       event that "caused" the anti-snap.
   * @param Object      user_info the user info passed to the objects.
   * @return boolean indicating if either the drag object or the 
   *                 target object accepted the anti-snap.
   */
  protected boolean make_anti_snap_active(
    snap_record snap_rec, 
    event       evt, 
    Object      anti_snap_info,
    Object      user_info)
    {
      boolean result;
      Point drag_pt, snap_pt, feature_pt;
      event evt_copy;

      /* record new active anti_snap */
      active_anti_snap = snap_rec;
      active_anti_snap_info = anti_snap_info;
      active_snap = null;

      /* put snap_pt into drag object's parent's coordinates */
      snap_pt = active_anti_snap.target_obj.local_to_global(
						     active_anti_snap.snap_pt);
      snap_pt = active_anti_snap.drag_obj.global_to_local(snap_pt);
      snap_pt = active_anti_snap.drag_obj.into_parent(snap_pt);

      /* account for offset of feature point within drag object . */
      feature_pt = active_anti_snap.drag_obj.feature_point(
					    active_anti_snap.drag_feature);
      snap_pt.x -= feature_pt.x;
      snap_pt.y -= feature_pt.y;

      /* make a copy of the event */
      evt_copy = new event(evt);

      /* anti-snap drag obj -- note reverse order from break */
      evt_copy.global_to_local(active_anti_snap.drag_obj);
      result = active_anti_snap.drag_obj.anti_snap_to(
        evt_copy, active_anti_snap.drag_pt, snap_pt, 
        active_anti_snap.drag_feature, active_anti_snap.target_obj,
        active_anti_snap_info, user_info);

      /* put drag_pt into target object's coordinates */
      drag_pt = active_anti_snap.drag_obj.into_local(active_anti_snap.drag_pt);
      drag_pt = active_anti_snap.drag_obj.local_to_global(drag_pt);
      drag_pt = active_anti_snap.target_obj.global_to_local(drag_pt);

      /* anti-snap target obj */
      evt_copy.global_to_local(active_anti_snap.target_obj);
      active_anti_snap.target_obj.anti_snap_from(
        evt_copy, drag_pt, active_anti_snap.snap_pt, 
        active_anti_snap.drag_obj, active_anti_snap.drag_feature, 
        active_anti_snap_info, user_info);

      return result;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Collect the set of snap target objects whose bounds are within snapping
   * distance of the bounding box of the feature points of the object being
   * dragged.  The set of objects is returned as a pick_collector object
   * loaded with the possible targets. 
   *
   * @param int            new_x    new left edge of the dragged object
   * @param int            new_y    new top edge of the dragged object
   * @param snap_draggable drag_obj the dragged object
   * @return pick_collector containing the set of candidate target objects.
   */
  protected pick_collector find_overlapped_targets(
    int new_x, int new_y, 
    snap_draggable drag_obj)
    {
      int             dx, dy;
      Point           feature_loc;
      Rectangle       feature_rect;
      top_level       root;
      pick_collector  bound_hits;

      /* compute global position of the feature point bounds */
      dx = new_x - drag_obj.x();
      dy = new_y - drag_obj.y();
      feature_loc = drag_obj.local_to_global(min_feature_x+dx,min_feature_y+dy);
      feature_rect = new Rectangle(feature_loc.x, feature_loc.y, 
				   feature_rect_w, feature_rect_h);

      /* expand the feature point bounds by the maximum snap distance in all 
       * directions.  this allows us to test for positions within snapping 
       * distance by simple overlap */
      feature_rect.x -= snap_dist(); 
      feature_rect.y -= snap_dist(); 
      feature_rect.width += snap_dist()*2;
      feature_rect.height += snap_dist()*2;

      /* find the root of our interactor tree */
      root = drag_obj.get_top_level();

      /* search interactor tree we are in for snap targets whose bounds are 
       * close enough to feature point bounding rectangle that they could
       * snap */
      bound_hits = new pick_collector();
      root.traverse_and_collect(bound_traversal(), 
				interactor_consts.TRAV_PICK, 
				snap_and_overlap, overlaps, rect_to_child, 
				feature_rect, bound_hits);
      return bound_hits;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Table of snaps that the user has rejected and disabled for this 
   *  drag.  This table contains snap_rec objects.
   */
  protected Hashtable rejected_snaps = new Hashtable();

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Attempt to find a snap point between some active feature point of the 
   * currently dragged object and some snap_targetable object.  If a snap
   * is found, appropriate calls in the snap_draggable and snap_targetable
   * protocols are made and true is returned.  (Protocol calls including 
   * unsnap_to() and unsnap_from(), or unanti_snap_to() and unanti_snap_from() 
   * for any active snaps or anti-snaps that no longer hold, as well as 
   * snap_to() and snap_from(), or anti_snap_to() and anti_snap_from() for 
   * new snaps or anti-snaps that become active).  If no suitable snap is 
   * found, false is returned.
   *
   * @param int            new_x     new x position of the dragged object.
   * @param int            new_y     new y position of the dragged object.
   * @param snap_draggable drag_obj  the dragged object.
   * @param event          evt,      the event "causing" the drag.
   * @param Object         user_info the user info to be passed.
   * @return boolean indicating whether a snap has been found.
   */
  protected boolean process_snap(
    int            new_x, 
    int            new_y, 
    snap_draggable drag_obj,
    event          evt,
    Object         user_info)
    {
      Point           feature_loc, snap_pt;
      pick_collector  bound_hits;
      Vector          possible_snaps;
      snap_targetable candidate;
      int             dx, dy;
      int             can, fpt;
      int_holder      dist2;
      snap_record     snap_rec;
      boolean         result;
      Object          anti_snap_info;

      /* find all snap targets who's bounds are within snapping distance of 
       * the bounding box of the drag object's active feature points. 
       * this limits us to high likelihood targets. */
      bound_hits = find_overlapped_targets(new_x, new_y, drag_obj);

      /* now do the full test on every object that was overlapped */
      dx = new_x - drag_obj.x();
      dy = new_y - drag_obj.y();
      possible_snaps = new Vector(bound_hits.num_picks());
      for (can = 0; can < bound_hits.num_picks(); can++)
	{
	  /* pull out the obj (we know it must be snap_targetable because of 
	   * the snap_and_overlap test in the traversal). */
	  candidate = (snap_targetable)bound_hits.pick(can).obj; 

	  /* consider each active feature point in dragged object */
	  for (fpt = 0; fpt < drag_obj.num_feature_points(); fpt++)
	    if (drag_obj.feature_point_enabled(fpt))
	      {
		/* pull out the active feature point and put it into coords of
		 * the candidate target */
		feature_loc = drag_obj.feature_point(fpt);
		feature_loc.x += dx;
		feature_loc.y += dy;
		feature_loc = drag_obj.local_to_global(feature_loc);
		feature_loc = candidate.global_to_local(feature_loc);
	          // note: to avoid xforming point again, could save it w/ pick
		  //       in find_overlapping_targets

		/* do the actual geometric test */
		dist2 = new int_holder();
		snap_pt = new Point(0,0);
		if (candidate.point_snaps(feature_loc, dist2, snap_pt))
		  {

		    /* its close enough, build a snap record for it */
		    snap_rec = new snap_record(drag_obj, fpt, candidate, 
				 dist2.value, new Point(new_x, new_y), snap_pt);

		    /* only act if the user hasn't already rejected this snap */
		    if (!rejected_snaps.containsKey(snap_rec))
		      {

		        /* insert in sorted list of candidates (list is 
			 * sorted by increasing distance) */
		        snap_rec.insert_in(possible_snaps);
		          // note: this ends up being an O(n^2) sort -- could be
		          //       a performance problem in which case we 
			  //       should just stick it in the vector here and 
			  //       sort later or use a tree instead of the 
			  //       vector
		      }
		  }
	      }
	}

      /* walk down geometric snaps starting at the closest */
      for (can = 0; can < possible_snaps.size(); can++)
	{
	  /* extract the snap record */
	  snap_rec = (snap_record)possible_snaps.elementAt(can);

	  /* test if the target will accept this snap semantically */
	  if (snap_rec.target_obj.snap_from_ok(snap_rec.snap_pt, 
				     snap_rec.drag_obj, snap_rec.drag_feature))
	    {
	      /* -- this is our snap -- */

	      /* if same snap is already active we do snap_feedback for it */
	      if (snap_rec.same_as(active_snap))
		return true;
		// do we need a snapped_feedback() ?? xx

	      /* if we have another active snap or anti-snap, break it */
	      break_active_snap(evt, user_info);
	      break_active_anti_snap(evt, user_info);

	      /* make the new snap */
	      return make_snap_active(snap_rec, evt, user_info);
	    }
	}

      /* if we get here, then all the snaps were rejected 
       * so now we see if we can do them as anti-snaps.  */

      /* walk down geometrically ok, but semantically rejected snaps, 
       * closest first */
      for (can = 0; can < possible_snaps.size(); can++)
	{
	  /* extract the snap record */
	  snap_rec = (snap_record)possible_snaps.elementAt(can);

	  /* test if the target will accept this anti-snap */
	  anti_snap_info = snap_rec.target_obj.anti_snap_from_ok(
	      snap_rec.snap_pt, snap_rec.drag_obj, snap_rec.drag_feature);
	  if (anti_snap_info != null)
	    {
	      /* -- found the anti-snap to do -- */

	      /* if same anti-snap is already active we are done */
	      if (snap_rec.same_as(active_anti_snap))
		return true;
		// do we need an anti_snapped_feedback()?? xx

	      /* if we have another active snap or anti-snap, break it */
	      break_active_snap(evt, user_info);
	      break_active_anti_snap(evt, user_info);

	      /* make the new anti-snap */
	      return make_anti_snap_active(snap_rec, evt, anti_snap_info, 
								  user_info);
	    }
	}

      /* if we get here, we have neither a snap nor an anti-snap
      
      /* if we had an existing snap or anti-snap, break it */
      break_active_snap(evt, user_info);
      break_active_anti_snap(evt, user_info);

      /* we go back with false to get a normal drag_feedback issued */
      return false;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Dispatch an event under this protocol.  This does full snap processing
   * including possible rejection and disabling of active snap points.
   *
   * @see #process_snap
   * @param evt       the event being dispatched (already approved by 
   *                  event_is_useful())
   * @param user_info an uninterpreted info object to be passed to the 
   *                  dragged object
   * @param to_obj    for focus agents this is always null
   * @param seq_num   for focus agents this is ignored
   * @return value indicating if the event was consumed 
   */
  public boolean dispatch_event(
    event      evt,
    Object     user_info,
    interactor to_obj,
    int        seq_num) 
{
      snap_draggable drag_obj;
      Object         drag_info;
      event          cur_event;
      boolean        result;
      Point          start_pt, snap_pt;
      int            x, y;

      /* extract our focus item and if there is none, bail out now */
      drag_obj = (snap_draggable)_focus_set.elementAt(0);
      if (drag_obj == null) return false;
      drag_info = _user_info_set.elementAt(0);

      /* make a local copy of the current event */
      cur_event = new event(evt);

      /* If this is end, clear the focus and let inform_focus_exit do work */
      if (evt.id() == event.MOUSE_UP)
	{
	  clear_focus(cur_event);
	  return true;
	}

      /* if this is a key to break the current snap do that */
      if (have_active() && evt.id() == event.KEY_PRESS &&
                           evt.key() == snap_break_key())
	{
	  /* break and reject any active snap */
	  if (have_active_snap())
	    {
	      rejected_snaps.put(active_snap, active_snap);
	      break_active_snap(evt, user_info);
	    }

	  /* break and reject any active anti-snap */
	  if (have_active_anti_snap())
	    {
	      rejected_snaps.put(active_anti_snap, active_anti_snap);
	      break_active_anti_snap(evt, user_info);
	    }

	  /* we let this fall through to provide a new position */
	}

      /* put event into coords of object we are dispatching to */
      cur_event.global_to_local(drag_obj);

      /* put start point into parent's coords */
      start_pt = drag_obj.global_to_local(start_x, start_y);
      start_pt = drag_obj.into_parent(start_pt);

      /* Compute our new position in parent */
      x = drag_obj.x_into_parent(cur_event.local_x()) - grab_x;
      y = drag_obj.y_into_parent(cur_event.local_y()) - grab_y;

      /* try to do a snap point for that x,y -- if no snap, do drag feedback */
      if (!process_snap(x,y, drag_obj, cur_event, drag_info))
	{
          return drag_obj.drag_feedback(cur_event, x, y,
			     start_pt.x, start_pt.y, grab_x, grab_y, drag_info);
	}
      else
	return true;
    }
}

/*=====================================================================*/

/** Helper interactor predicate class that tests if an object implements 
 *  the snap_targetable interface. */
class is_snap_targetable implements interactor_pred {
  /** 
   * Perform the predicate test.
   *  
   * @param obj        the interactor we testing.
   * @param parameters ignored for this predicate 
   * @return whether the object implements the snap_targetable interface.
   */
  public boolean test(interactor obj, Object parameters) 
    {
      return (obj instanceof snap_targetable);
    }
}

/*=====================================================================*/

/** Helper class the holds information about a candidate snap */
class snap_record {

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The object being dragged */
  public snap_draggable drag_obj;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The feature point index within the drag object that is snapping */
  public int drag_feature;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The target object being snapped to */
  public snap_targetable target_obj;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The square of the distance from the nominal drag point to the snap */
  public int dist2;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The nominal drag point (where we would be w/o a snap).  This is the
   *  drag object's parent's coordinates.  */
  public Point drag_pt;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** The snap location in the target object's coordinates. */
  public Point snap_pt;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Build a new snap_record. 
   *
   * @param snap_draggable  drg_obj     the dragged object.
   * @param int             drg_feature the snapped feature point within that 
   *                                    object.
   * @param snap_targetable targ_obj    the target object.
   * @param int             dst2        the square of the distance to the snap 
   *                                    point.
   * @param Point           drg_pt      where the dragged object would have 
   *                                    been without the snap (in the parent's 
   *                                    coordinates).
   * @param Point           snp_pt      location we are snapping to in the 
   *                                    target object's coordinates.
   */
  snap_record(
    snap_draggable  drg_obj, 
    int             drg_feature,
    snap_targetable targ_obj,
    int             dst2,
    Point           drg_pt,
    Point           snp_pt)
    {
      drag_obj     = drg_obj;
      drag_feature = drg_feature;
      target_obj   = targ_obj;
      dist2        = dst2;
      drag_pt      = drg_pt;
      snap_pt      = snp_pt;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Comparison operator for sorting snap records.  Records are sorted
   * by their dist2 field.
   *
   * @param snap_record other_rec the snap record we are comparing against. 
   * @return boolean indicating if we are to be considered less then 
   *                 (closer than) the given snap record.
   */
  public boolean less_than(snap_record other_rec)
    {
      return dist2 < other_rec.dist2;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Insert this snap record into a sorted Vector of snap records. Where
   * sorting criteria is defined by the less_than() method and the vector is
   * sorted lowest to highest.  
   *
   * @param Vector sorted_vector already sorted vector of snap records that we
   *                             inserting into.
   */
  public void insert_in(Vector sorted_vector)
    {
      int         i;
      snap_record other;

      /* do a simple linear search for the insert point */
      for (i = 0; i < sorted_vector.size(); i++)
	{
	  /* if we are smaller than next one, insert here */
	  other = (snap_record)sorted_vector.elementAt(i);
	  if (this.less_than(other))
	    {
	      /* do the insert and we are done */
	      sorted_vector.insertElementAt(this, i);
	      return;
	    }
	}

       /* if we get here, we have run off the end and we append it */
       sorted_vector.addElement(this);

       /*  Note: this ends up being an O(N) insert which leads to an O(N^2)
        *        overall sort.  This could be a performance problem, in which
        *        case we probably need to use something other than a vector.
	*/
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Determine if this is the same snap as another.  Two snaps are considered
   * the same if they are between the same drag_obj/feature-point and target
   * object (i.e., we ignore position and distance).
   * @param snap_record other_rec the snap record we are comparing against.
   */
  public boolean same_as(snap_record other_rec)
    {
      return other_rec != null &&
             drag_obj     == other_rec.drag_obj && 
             drag_feature == other_rec.drag_feature &&
             target_obj   == other_rec.target_obj;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Equality for hash lookup (same semantics as same_as()). */
  public boolean equals(Object other_rec) 
    {
      if (!(other_rec instanceof snap_record))
	return false;
      else
        return same_as((snap_record)other_rec);
     }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Hash code generation */
  public int hashCode() 
    {
      return drag_obj.hashCode() ^ drag_feature ^ target_obj.hashCode();
    }
    
  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Produce a human readable string for dumping the snap_record */
  public String toString()
    {
      return
	"-- snap_record --\n" + 
	"  drag_obj=" + drag_obj.getClass().getName() + 
	": " + drag_obj.hashCode() + " " +
	"  drag_feature=" + drag_feature + "\n" +
	"  target_obj=" + target_obj.getClass().getName() + 
	": " + target_obj.hashCode() + "\n" +
	"  dist2 = " + dist2 + 
	" drag_pt=" + drag_pt.x + "," + drag_pt.y +
	" snap_pt=" + snap_pt.x + "," + snap_pt.y + "\n" +
	"----";
    }
}


/*=====================================================================*/
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
