package sub_arctic.lib;

import sub_arctic.input.callback_object;
import sub_arctic.constraints.std_constraint_consts;

import java.awt.Frame;
import java.awt.Event;
import java.awt.Dimension;
import java.applet.Applet;
import java.awt.Component;

/**
 * This class allows you to put up an external frame with a sub arctic
 * interface in it.   Note that this is an AWT container, so if you want
 * its size, location, etc. you have to use the AWT calls.<P>
 * 
 * Since this is an AWT frame you'll need to call dispose to release
 * the resource for this object ('hide' just gets it off screen) unless
 * the user does the "go away" box, in which case we dispose of it. <P>
 *
 * This object supports a callback. It will get called with a callback_num
 * of 0 and a null callback_object. The interactor making the call
 * will be the top_level of this frame. This callback is called when
 * the user closes the window (only). Note that the callback is called 
 * with a null event. We don't have a sub arctic event at the time of 
 * the callback!<P>
 *
 * Subtle point about inside-out interfaces: If you want an "inside-out"
 * type of sizing behavior you should be aware of this: If you use 
 * constraints on your top_level to compute the inside-out layout
 * (and be careful to insure you don't get cycles) they will be 
 * REMOVED when you call show on this object.  The last value will 
 * preserved, so the initial size will be whatever the result of the
 * constraints were; we remove the constraints because the size of
 * your top_level is tied to the size of the AWT component it is 
 * living in and these components get (stupidly) resized a couple of
 * times (including going through zero!) before the interface ends up
 * on the screen. Thus, you would get an exception because you would
 * end up trying to assign to a constrained value. Trust me on this one
 * thing: If you use constraints to compute your inside-out interface
 * and then use the "setResizable" method to not allow the user to
 * resize your interface, everything will work out.<P>
 *
 * If you are using this object to make a window for an already
 * constructed interface (i.e you supply a non-null top_level)
 * none of the normal <KBD>build_ui</KBD> type of methods will
 * get called. We assume you are already "done" with UI
 * construction. If you want to build the UI after this frame
 * is constructed, pass a null value to any of the constructors of
 * this class and we will go through the same sequence as
 * interactor_applet for constructing the interface (pre_build_ui())
 * build_ui() and then post_build_ui()) <I>at show time</I>.  
 * That is to say that your build_ui() type methods will not 
 * get called until the object is shown. We do this to allow the
 * object's constructor to complete before we run the code
 * to construct the UI. If you want to force an interface build
 * you can call initialize() yourself.<P>
 * 
 * If you use the approach of building the UI in the build_ui
 * methods, & don't explicitly set a size, the size of the frame 
 * will be computed based on the size of your top_level 
 * after the call to post_build_ui().<P>
 * 
 * @author Ian Smith
 */

public class interactor_frame extends Frame implements std_constraint_consts{
 /* 
  * XXX AWT seems to have a bogosity in that the size you "reshape" 
  * a frame to is not always the size you get!  Workaround? Seems
  * to be tied to the window manager... XXX  is this an inset problem? [SEH]
  */

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Storage for the frame's size (width). We use -1 as a marker
   * to know if this has been set or not.
   */
  private int _forced_width=-1;

  /**
   * Storage for the frame's size (height). We use -1 as a marker
   * to know if this has been set or not.
   */
  private int _forced_height=-1;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Callback object.
   */
  protected callback_object _obj;

  /**
   * Retrieve the callback object associated with this interactor_frame.
   * @return callback_object the object this frame makes callbacks on
   */
  public callback_object callback_obj() { return _obj;}

  /**
   * Change the object we will make the callbacks on.
   * 
   * @param callback_object o the new callback target
   */
  public void set_callback_obj(callback_object o){ _obj=o;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Storage and accessor for the toplevel. 
   */
  protected top_level _top;

  /**
   * Access the top level interactor for this frame.
   * 
   * @return top_level this interactor_frame's top level
   */
  public top_level top_interactor() { return _top;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Initialize this object. This assumes that you have already
   * set up the _forced_width and _forced_height variables
   * before you call this function.
   * 
   * @param top_level         tlevel the new top_level for this frame
   * @param interactor_applet a      the applet to use instead of making a new 
   *                                 canvas (pass null for the applet if you 
   *                                 want us to create a canvas for you).
   */
  private void init(top_level tlevel,interactor_applet a) {
    Component c;
    interactor_canvas ic;

    /* if they don't have an applet, build the canvas for them */
    if (a == null) {

      /* make the applet and install a top level as its interface */
      ic=new interactor_canvas(_forced_width,_forced_height);
      ic.set_top_interactor(tlevel);
      c=ic; /* make a copy for convenience later */

    } else {

      /* they do have an applet, use it */
      c = a;

    }

    /* the component assuming a default layout manager of flow */
    add("Center",c);

    /* set the toplevel */
    _top=tlevel;

    /* resize to fit */
    if (a==null) {
      resize(_forced_width, _forced_height);
    } else {
      resize(_forced_width, _forced_height);
      a.resize(_forced_width, _forced_height);
      a.validate();
    }
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
    
  /**
   * Construct an interactor_frame. This object will be initially set
   * up to have a certain size. If you supply null as the top_level
   * we will go through the sequence pre_build_ui(), build_ui()
   * and post_build_ui() to allow you to construct the interface.
   * Note that the size of the resulting frame will be the
   * values supplied to this function independent of the size of
   * the constructed top_level.
   * 
   * @param String    title the title for this window when it gets to the screen
   * @param int       fw    the width of the new frame
   * @param int       fh    the height of the new frame
   * @param top_level tl    the top_level to put in this frame
   */
  public interactor_frame(String title, int fw, int fh,top_level tl) {

    /* tell the frame what's up */
    super(title);

    manager.register_awt_component(this);
    addNotify();

    /* initialize the size here*/
    _forced_width=fw;
    _forced_height=fh;

    /* is the top_level interactor null, if so we need to allow them
     * a way to build the interface.... for now we just mark ourselves
     * as not built
     */
    if (tl==null) {
      _ui_constructed=false;
      return;
    }

    /* they gave us a top level, go ahead and do the real init */
    init(tl, null);
  }

   //had:
   //* @exception general PROPAGATED from constructor of top_level

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Construct an interactor_frame from a top_level.  We deduce the
   * size of the interactor_frame from the top_level, so be sure
   * have your top_level sized correctly (probably using constraints)
   * or this object will end up with the wrong size.   If you pass
   * null here for the top_level, we go through the usual sequence
   * of calls to pre_build_ui(), buid_ui() and then post_build_ui().
   * That sequence will be called at the time you call show() or
   * you can do it explicitly with initialize().<P>
   * 
   * After that sequence we compute the size of your top_level 
   * and use that to size the frame.  Note: Initially your top_level
   * is sized to be 10x10 pixels, so if you don't want your interface
   * to be that size, you should size it in one of these calls or
   * install constraints on it.<P>
   *
   * This ends up calling the static routine on interactor_canvas to
   * derive the size and remove the constraints.
   *  
   * @param String    title the title for this window on the screen
   * @param top_level tl    the top_level to put in this frame
   */
  public interactor_frame(String title, top_level tl) {

    super(title);

    Dimension d;
    manager.register_awt_component(this);
    addNotify();

    /* is it null? */
    if (tl==null) {
      _ui_constructed=false;
      return;
    }
      
    /* figure out how big the object is */
    d=interactor_canvas.size_from_top_level(tl);
    _forced_width=d.width;
    _forced_height=d.height;

    /* initialize it */
    init(tl,null);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** Callback number constant for the close of window callback. */
  public static final int CLOSE_WINDOW_CALLBACK = 0;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Hook for you to have your own close window processing. If you
   * return true here the window goes away, otherwise the window
   * stays on screen.
   * 
   * return boolean true if you want the window to go away
   */
  public boolean close_window() {
    /* if there is a callback object */
    if (callback_obj()!=null) {

      /* tell it what happened */
      /* NOTE: We FAKE the application out by giving it the toplevel 
       * for this frame, but we can't fake any sub_arctic events, we
       * don't have one! */  //xx MUST PASS AN EVENT HERE!!
      callback_obj().callback(top_interactor(),null,CLOSE_WINDOW_CALLBACK,null);
    }
    return true; /* default to allowing close */
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is a little (AWT) event handler for the frame. It shouldn't
   * be called except to deal with events that don't get handled by
   * SubArctic.<p>
   *
   * It calls through to destroy_window()... if that function returns
   * true then it handles the event and the window closes and gets
   * destroyed. Otherwise it does nothing.   
   * @param Event event the event we are handling (this an AWT Event, not 
   *                    sub_arctic event).
   * @return true if we handled the event
   */
  public boolean handleEvent(Event event) {
    if (event.id == Event.WINDOW_DESTROY) {
      if (close_window()) { //xx pass event in here!
	hide();
	dispose();
      }
    }   
    return super.handleEvent(event);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is part of the AWT world. It is supposed to be called
   * by layout manager's to ask us how big we would like to be. We just
   * tell it our minimum size. <P>
   * 
   * Its not clear when, where, why, or if this function will actually 
   * be called by a layout manager.  <P>
   * 
   * @return Dimension the dimensions we prefer for this frame.
   */
  public Dimension preferredSize() {
    return minimumSize();
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is part of the AWT world. It is supposed to be called
   * by layout manager's to ask how what the minimal amount of screen
   * space we can live with is.We return the size that the user initially
   * gave us when this object was created. <P>
   * 
   * Its not clear when, where, why, or if this function will actually 
   * be called by a layout manager. <P>
   *
   * @return Dimension the minimum dimension we can live with
   */
  public Dimension minimumSize() {
    return new Dimension(_forced_width, _forced_height);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called to allow you to initialize data structures
   * that are needed for building the UI. By default, it does nothing.
   */
  public void pre_build_ui() {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called to allow you to process the now 
   * constructed interactor tree after its built.
   * 
   * @param base_parent_interactor t root of the newly constructed tree
   */
  public void post_build_ui(base_parent_interactor t) {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function is called to create an install a top_level 
   * interactor in this applet. By default it creates a toplevel
   * of the same size as the interactor and then installs  it.
   * 
   * @param int w the width of the new top_level to construct
   * @param int h the height of the new top_level
   */
  public void make_top_level(int w, int h) {
    _top=new top_level(0,0,w,h);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function gets called so you can construct your
   * user interface. The top_level provided is already installed
   * as the root of your interactor tree for this applet.
   * By default, this function does nothing.
   *
   * @param base_parent_interactor top the new root interactor for this applet
   */
  public void build_ui(base_parent_interactor top) {}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This is a convenience function for replicating applets in
   * frames. If you use this method, this method will call init() on 
   * your applet (so the applet will get the associated pre_build_ui(),
   * build_ui() and post_build_ui() functions) and the interactor
   * frame will <B>not</B> get he build_ui() calls. You have to
   * supply a size for this applet because applets expect to be
   * given a size via their browser.  Your applet might get confused 
   * if you have already called init() on it before you call 
   * this constructor. 
   * 
   * @param String n the name of the frame
   * @param interactor_applet a the applet to insert in this frame
   * @param int w the width of this frame
   * @param int h the height of this frame
   */
  public interactor_frame(String n, interactor_applet a, int w, int h) {
    super(n);
    manager.register_awt_component(this);
    addNotify();
    a.init();
    _forced_width=w;
    _forced_height=h;
    init(null,a);
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * hold onto whether or not the UI is built
   */
  protected boolean _ui_constructed=true;

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Determine if the user interface has been constructed.
   * @return boolean true if the UI has been built
   */
  public boolean ui_constructed() { return _ui_constructed;}

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * This function gets called to force the UI to get built if
   * there is no top_level and the UI has not been previously
   * built. Calls other than the first one to initialize() don't
   * have any effect. This function gets called automatically
   * by the show() routine.
   */
  public void initialize() {
    top_level tl;
    Dimension d;

    if (ui_constructed()==false) {
      _ui_constructed=true;
      /* if they gave us a height, use that... */
      if (_forced_width!=-1) {
	make_top_level(_forced_width,_forced_height);  // make it 
      } else {
	/* guess a size ...we hope they override it later but
	   if they don't at least they'll get something*/
	make_top_level(300,200);
      }

      pre_build_ui();

      /* steal the top level (again) back out of _top */
      tl=_top;
      build_ui(tl);  // build the UI
      post_build_ui(tl);

      /* have they set it already ? */
      if (_forced_width!=-1) {
	/* they have explicitly set it */
	init(tl,null);
	return;
      }

      /* they didn't set it.... figure out how big the object is */
      d=interactor_canvas.size_from_top_level(tl);

      /* send that info to init */
      _forced_height=d.height;
      _forced_width=d.width;
      init(tl,null);
    }
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * We override the show() method to make sure the sub_arctic
   * interface is constructed.
   */
  public void show() {
    initialize();// double check!
    super.show();
  }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
}
/*=========================== COPYRIGHT NOTICE ===========================

This file is part of the subArctic user interface toolkit.

Copyright (c) 1996 Scott Hudson and Ian Smith
All rights reserved.

The subArctic system is freely available for most uses under the terms
and conditions described in 
  http://www.cc.gatech.edu/gvu/ui/sub_arctic/sub_arctic/doc/usage.html 
and appearing in full in the lib/interactor.java source file.

The current release and additional information about this software can be 
found starting at: http://www.cc.gatech.edu/gvu/ui/sub_arctic/

========================================================================*/
